@@ -1,7 +1,8 @@
-from math import ceil, sqrt
-from itertools import product, islice, count
-from .arbitrary import Arbitrary, MappedArbitrary, FilteredArbitrary
-from .utils import filter_possible, unique
+from collections import namedtuple
+from itertools import islice
+from .arbitrary import Arbitrary, MappedArbitrary, FilteredArbitrary, \
+    get_arbitrary, with_samples
+from .utils import filter_possible, unique, product
 from .numbers import int_
 from .primitives import char, letter
 
@@ -12,12 +13,14 @@ __all__ = ['tuple_',
            'str_', 'str_letters']
 
 
+@Arbitrary.set_for(tuple)
+@with_samples
 class tuple_(Arbitrary):
     """ Tuple with specified elements. """
 
-    @Arbitrary.args_for_sample(int_, letter)
-    def __init__(self, *arbitraries):
-        self.arbitraries = arbitraries
+    @Arbitrary.args_for_sample((int_, letter))
+    def __init__(self, arbitraries):
+        self.arbitraries = list(map(get_arbitrary, arbitraries))
 
     def could_generate(self, x):
         return all(arb.could_generate(y)
@@ -26,9 +29,8 @@ class tuple_(Arbitrary):
     def next_random(self):
         return tuple(arb.next_random() for arb in self.arbitraries)
 
-    def gen_serial(self, amount):
-        inner_amount = int(ceil(sqrt(amount)))
-        return product(*[arb.gen_serial(inner_amount)
+    def gen_serial(self):
+        return product(*[arb.gen_serial()
                          for arb in self.arbitraries])
 
     def shrink(self, x):
@@ -37,14 +39,40 @@ class tuple_(Arbitrary):
                 yield x[:i] + (shr_y,) + x[i + 1:]
 
 
+@Arbitrary.set_for(namedtuple)
+class namedtuple_(MappedArbitrary):
+    def __init__(self, typename, fields):
+        self.tupletype = namedtuple(typename, [name for name, _ in fields])
+        self.inner_arb = tuple_([value for _, value in fields])
+
+    def pack(self, tup):
+        return self.tupletype._make(tup)
+
+    def unpack(self, ntup):
+        return tuple(ntup)
+
+
+class dict_fixkeys(MappedArbitrary):
+    def __init__(self, **items):
+        self.inner_arb = namedtuple_('DictItems', items.items())
+
+    def pack(self, ntup):
+        return dict(ntup._asdict())
+
+    def unpack(self, dct):
+        ttype = self.inner_arb.tupletype
+        return ttype._make([dct[name] for name in ttype._fields])
+
+
 @Arbitrary.set_for(list)
+@with_samples
 class list_(Arbitrary):
     """ List with configurable length and elements. """
 
-    @Arbitrary.args_for_sample(length=int_(0, 5), elements=int_(-100, 100))
+    @Arbitrary.args_for_sample(length=int_(0, 10), elements=int_(-100, 100))
     def __init__(self, length=int_(0, 20), elements=int_):
-        self.length = length
-        self.elements = elements
+        self.length = get_arbitrary(length)
+        self.elements = get_arbitrary(elements)
 
     def could_generate(self, x):
         return self.length.could_generate(len(x)) and \
@@ -56,9 +84,9 @@ class list_(Arbitrary):
 
     @unique
     @filter_possible
-    def gen_serial(self, amount):
-        def gen_for_depth(depth, amount):
-            for l in self.length.gen_serial(amount):
+    def gen_serial(self):
+        def gen_for_depth(depth):
+            for l in self.length.gen_serial():
                 if not (depth >= l or depth == 0):
                     continue
 
@@ -66,14 +94,12 @@ class list_(Arbitrary):
                     yield []
                 else:
                     el_amount = depth + 1
-                    elems = self.elements.gen_serial(el_amount)
+                    elems = list(islice(self.elements.gen_serial(), el_amount))
                     for res in product(elems, repeat=l):
                         yield list(res)
 
-        inner_amount = int(ceil(sqrt(amount)))
-
-        for depth in count():
-            for item in gen_for_depth(depth, inner_amount):
+        for depth in range(50):
+            for item in gen_for_depth(depth):
                 yield item
 
     @filter_possible
@@ -95,6 +121,7 @@ class list_(Arbitrary):
                 yield [el]
 
 
+@with_samples
 class list_ordered(MappedArbitrary):
     """ List where elements are in order.
 
@@ -103,6 +130,7 @@ class list_ordered(MappedArbitrary):
         Non-unique serially-generated values and shrink results are possible.
     """
 
+    @Arbitrary.args_for_sample(length=int_(0, 10), elements=int_(-100, 100))
     def __init__(self, length=int_(0, 20), elements=int_,
                  key=lambda x: x, reverse=False):
         self.inner_arb = list_(length=length, elements=elements)
@@ -116,6 +144,7 @@ class list_ordered(MappedArbitrary):
         return lst
 
 
+@with_samples
 class list_unique(FilteredArbitrary):
     """ List with unique elements.
 
@@ -126,6 +155,7 @@ class list_unique(FilteredArbitrary):
         Also, upper length bound is never reached in some cases.
     """
 
+    @Arbitrary.args_for_sample(length=int_(0, 10), elements=int_(-100, 100))
     def __init__(self, length=int_(0, 20), elements=int_):
         self.inner_arb = list_(length=length, elements=elements)
 
@@ -134,6 +164,7 @@ class list_unique(FilteredArbitrary):
 
 
 @Arbitrary.set_for(set)
+@with_samples
 class set_(MappedArbitrary):
     """ Set of unique elements.
 
@@ -159,6 +190,7 @@ class set_(MappedArbitrary):
 
 
 @Arbitrary.set_for(dict)
+@with_samples
 class dict_(MappedArbitrary):
     """ Dictionary with specified items (key-value pairs) arbitrary.
 
@@ -172,7 +204,7 @@ class dict_(MappedArbitrary):
                  items=None,
                  keys=int_, values=int_):
         if items is None:
-            items = tuple_(keys, values)
+            items = tuple_((keys, values))
         self.inner_arb = list_(length=length, elements=items)
 
     def pack(self, lst):
@@ -183,6 +215,7 @@ class dict_(MappedArbitrary):
 
 
 @Arbitrary.set_for(bytearray)
+@with_samples
 class bytearray_(MappedArbitrary):
     """ Mutable bytearray with configurable length and elements. """
 
@@ -196,6 +229,7 @@ class bytearray_(MappedArbitrary):
         return list(barr)
 
 
+@with_samples
 class bytes_(MappedArbitrary):
     """ Immutable bytearray with configurable length and elements.
 
@@ -219,6 +253,7 @@ if bytes is not str:
 
 
 @Arbitrary.set_for(str)
+@with_samples
 class str_(MappedArbitrary):
     """ String with configurable length and characters. """
 
@@ -232,6 +267,7 @@ class str_(MappedArbitrary):
         return list(s)
 
 
+@with_samples
 class str_letters(str_):
     """ String consisting of letters only. """
 
